#include <SPI.h>
#include <SdFat.h>
#include <SdFatUtil.h> 
#include <SFEMP3Shield.h>
#include <Servo.h>            // Servo libary(comes with Arduino)

Servo servoLeft;              // Define left servo
Servo servoRight;             // Define right servo
SFEMP3Shield MP3player;

byte lastPlayed = 255;

void setup() {
  
  Serial.begin(9600);
  
  servoLeft.attach(10);         // Left servo to pin D10
  servoRight.attach(5);         // Right sevo to pin D5 
 
  pinMode(A0, INPUT);           // Set pin A0 to input
  pinMode(A1, INPUT);           // Set pin A1 to input
  pinMode(A2, OUTPUT);          // Set pin A2 to output
  digitalWrite(A0, HIGH);       // Activate A0 pullup resistor
  digitalWrite(A1, HIGH);       // Activate A1 pullup resistor
  digitalWrite(A2, LOW);        // Make low for switch ground

  MP3player.begin();
  MP3player.SetVolume(10, 10);
  Serial.println ("MP3 started");
 
}

void loop() {

  if (digitalRead(A0) == 0) {           // If right bumper hit
    triggerTrack(A1, "track001.mp3");   // Play this track
    reverse();                          // Reverse robot for 400 ms
    delay(400); 
    spinLeft();                         // Spin left
    while(MP3player.isPlaying()) {      // Wait while clip is played
      delay(1);
    }
    forward();                          // Go forward again
    Serial.println("pbRight");          // Display statuis
  }

  if (digitalRead(A1) == 0) {          // If left bumper hit
    triggerTrack(A0, "track002.mp3");
    reverse();
    delay(400); 
    spinRight();                       // Spin right
    while(MP3player.isPlaying()) {
      delay(1);
    }
    forward();
    Serial.println("pbLeft");
  }
}

// Trigger Track and motion routines
void triggerTrack(byte pin, char* track) {
  // Test if an MP3 is playing, and trigger if not the same as before
  if (lastPlayed != pin && MP3player.isPlaying()) {
    MP3player.stopTrack();       // Stop track if another is playing
  }
  MP3player.playMP3(track);      // Play the new track
  lastPlayed = pin;              // Set this pin number as last played
}

// Motion routines for forward, reverse, turns, and stop
void forward() {
  servoLeft.write(180);
  servoRight.write(0);
}
void reverse() {
  servoLeft.write(0);
  servoRight.write(180);
}
void spinLeft() {
  servoLeft.write(0);
  servoRight.write(0);
}
void spinRight() {
  servoLeft.write(180);
  servoRight.write(180);
}
void stopRobot() {
  servoLeft.write(90);
  servoRight.write(90);
}
