#include <SPI.h>
#include <SdFat.h>
#include <SdFatUtil.h> 
#include <SFEMP3Shield.h>
#include <Servo.h>            // Servo libary(comes with Arduino)
#include <IRremote.h>         // Infrared remote library; see text
#include <IRremoteInt.h>

Servo servoLeft;              // Define left servo
Servo servoRight;             // Define right servo
SFEMP3Shield MP3player;

const int RECV_PIN = A5;     // Receiver input pin on A5
IRrecv irrecv(RECV_PIN);     // Define IR recever object
decode_results results;

byte lastPlayed = 255;

void setup() {
  
  Serial.begin(9600);
  
  servoLeft.attach(10);      // Left servo to pin D10
  servoRight.attach(5);      // Right sevo to pin D5 
 
  pinMode(A0, INPUT);        // Set pin A0 to input
  pinMode(A1, INPUT);        // Set pin A1 to input
  pinMode(A2, OUTPUT);       // Set pin A2 to output
  digitalWrite(A0, HIGH);    // Activate A0 pullup resistor
  digitalWrite(A1, HIGH);    // Activate A1 pullup resistor
  digitalWrite(A2, LOW);     // Make low for switch ground
  
  pinMode(A4, OUTPUT);       // IR power, ground pins
  pinMode(A3, OUTPUT);
  digitalWrite(A4, LOW);     // IR ground
  digitalWrite(A3, HIGH);    // IR power
  irrecv.enableIRIn();       // Start the receiver

  MP3player.begin();
  MP3player.SetVolume(10, 10);
  Serial.println ("MP3 started");
 
}

void loop() {
 
  // Make tracks 000 and 001 slightly longer (about 1 second)
  // Make the other clips shorter, 1/2 second or less
  if (digitalRead(A0) == 0) {           // If right bumper hit
    triggerTrack(A1, "track000.mp3");   // Play this track; triggers as #14
    reverse();                          // Reverse robot for 400 ms
    delay(400); 
    spinLeft();                         // Spin left
    while(MP3player.isPlaying()) {      // Wait while clip is played
      delay(1);
    }
    forward();                          // Go forward again
    Serial.println("pbRight");          // Display statuis
  }

  if (digitalRead(A1) == 0) {          // If left bumper hit
    triggerTrack(A0, "track001.mp3");  // Triggers as #15
    reverse();
    delay(400); 
    spinRight();                       // Spin right
    while(MP3player.isPlaying()) {
      delay(1);
    }
    forward();
    Serial.println("pbLeft");
  }
  
   if (irrecv.decode(&results)) {  // If valid value was received
     switch (results.value) {      // Match button against Sony TV codes
       case 0x10:
         Serial.println("1");
         break;
       case 0x810:
         triggerTrack(2, "track002.mp3");
         forward();
         Serial.println("2");
         break;
       case 0x410:
         Serial.println("3");
         break;
       case 0xC10:
         triggerTrack(5, "track003.mp3");
         spinLeft();
         Serial.println("4");
         break;
       case 0x210:
         triggerTrack(5, "track004.mp3");
         stopRobot();
         Serial.println("5");
         break;
       case 0xA10:
         triggerTrack(5, "track005.mp3");
         spinRight();
         Serial.println("6");
         break;
       case 0x610:
         Serial.println("7"); 
         break;
       case 0xE10:
         triggerTrack(8, "track006.mp3");
         reverse();
         Serial.println("8");
         break;
       case 0x110:
         Serial.println("9");
         break;
      }        
    irrecv.resume();         // Receive the next value
    delay(10);               // 10ms delay
  }
  
}

void triggerTrack(byte pin, char* track) {
  Serial.println(pin);
  // Test if an MP3 is playing, and trigger if not the same as before
  if (lastPlayed != pin && MP3player.isPlaying()) {
    MP3player.stopTrack();   // Stop track if another is playing
  }
  MP3player.playMP3(track);  // Play the new track
  lastPlayed = pin;          // Set this pin number as last played
}

// Motion routines for forward, reverse, turns, and stop
void forward() {
  servoLeft.write(180);
  servoRight.write(0);
}
void reverse() {
  servoLeft.write(0);
  servoRight.write(180);
}
void spinLeft() {
  servoLeft.write(0);
  servoRight.write(0);
}
void spinRight() {
  servoLeft.write(180);
  servoRight.write(180);
}
void stopRobot() {
  servoLeft.write(90);
  servoRight.write(90);
}
